// ///////////////////  ***********************  \\\\\\\\\\\\\\\\\\\\ //
//
//  SoSDLRenderArea, version 1.0
//
//  a simple window component like SoXtRenderArea for Inventor
//
//  Copyright (C) 2001-2002 by PC John (pcjohn@email.cz). 
//  All rights reserved.
//
//  This game is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public License
//  version 2.1 as published by the Free Software Foundation. See the
//  file LICENSE.LGPL at the root directory of the distribution for
//  more details.
//
// \\\\\\\\\\\\\\\\\\\  ***********************  //////////////////// //


#ifndef SO_SDL_RENDER_AREA_H_
#define SO_SDL_RENDER_AREA_H_


#define SoXt SoSDL
#define SoXtRenderArea SoSDLRenderArea

#include <Inventor/SoSceneManager.h>
#include <Inventor/SbViewportRegion.h>
#include <Inventor/events/SoKeyboardEvent.h>
#include <Inventor/events/SoLocation2Event.h>
#include <Inventor/events/SoMouseButtonEvent.h>
#include <Inventor/actions/SoGLRenderAction.h>
#include <Inventor/fields/SoSFTime.h>
#include <SDL/SDL.h>


class SoSDL;
class SoSDLRenderArea;


typedef SDL_Surface* Widget;



class SoSDL
{
friend SoSDLRenderArea;
public:

  static Widget init(const char *appName, const char *className = "Inventor");
  static void done();
  static void mainLoop();

  static void show(Widget widget = NULL);
  static void hide(Widget widget = NULL);

private:

  static bool processSensors(void);
  static void sensorQueueChangedCallback(void *data);
  static unsigned int postWakeUpMessageCallback(unsigned int interval, void *userData);
  
  static SoSDLRenderArea *windowsList;
  static bool widgetVisible;

  static void addWindow(SoSDLRenderArea *window);
  static void removeWindow(SoSDLRenderArea *window);

  static SoSDLRenderArea* getWindowFromHandle(Widget id);

  static bool stop;

};



class SoSDLRenderArea
{
friend SoSDL;

public:

  SoSDLRenderArea(Widget parent = NULL,
    const char *name = NULL,
    SbBool buildInsideParent = TRUE,
    SbBool getMouseInput = TRUE,
    SbBool getKeyboardInput = TRUE);
  ~SoSDLRenderArea();

  // SoSDL extensions
  void setFullScreenParams(int width, int height, int bpp = 0)  { fsWidth = width; fsHeight = height; fsBPP = bpp; }
  void getResolutionParams(int *width, int *height, int *bpp) const { if (width) *width = fsWidth; if (height) *height = fsHeight; if (bpp) *bpp = fsBPP; }
  void setFullScreen(bool b);
  bool isFullScreen() const { return fullScreen; }

  typedef struct { int width; int height; int bpp; } Resolution;
  static int getResolutionList(const Resolution *&resList, int depthBits = 0);
  static bool isResolutionSupported(int width, int height, int bpp = 0);
  bool isResolutionSupported() const { return isResolutionSupported(fsWidth, fsHeight, fsBPP); }

  // supported by SoGlut but no by SDL:
  // leaved only for API compatibility
  void setPosition(SbVec2s pos)  {}
  SbVec2s getPosition() const { return SbVec2s(0,0); }
  void centerWindow()  {}

  // unsupported from SoXtRenderArea
  //void setEventCallback(SoXtRenderAreaEventCB *fcn, void *userData = NULL)

  // from SoXtRenderArea
  virtual void setSceneGraph(SoNode *newScene)  { sceneManager->setSceneGraph(newScene); };
  virtual SoNode* getSceneGraph()  { return sceneManager->getSceneGraph(); };

  void setBackgroundColor(const SbColor &c)  { sceneManager->setBackgroundColor(c); };
  const SbColor& getBackgroundColor() const  { sceneManager->getBackgroundColor(); };

  void setViewportRegion(const SbViewportRegion &newRegion);
  const SbViewportRegion& getViewportRegion() const;

  void setTransparencyType(SoGLRenderAction::TransparencyType type);
  SoGLRenderAction::TransparencyType getTransparencyType() const;

  void setAntialiasing(SbBool smoothing, int numPasses)  { sceneManager->setAntialiasing(smoothing, numPasses); };
  void getAntialiasing(SbBool &smoothing, int &numPasses) const  { sceneManager->getAntialiasing(smoothing, numPasses); };

  void setClearBeforeRender(SbBool trueOrFalse, SbBool zbTrueOrFalse=TRUE)
    { clearColorBuffer = trueOrFalse; clearZBuffer = zbTrueOrFalse; };
  SbBool isClearBeforeRender() const  { return clearColorBuffer; };
  SbBool isClearZBufferBeforeRender() const  { return clearZBuffer; };

  void setAutoRedraw(SbBool trueOrFalse)  { trueOrFalse ? autoRedraw = true : autoRedraw = false; };
  SbBool isAutoRedraw() const  { return autoRedraw; };
  void setRedrawPriority(unsigned long priority)  { sceneManager->setRedrawPriority(priority); };
  unsigned long getRedrawPriority() const  { return sceneManager->getRedrawPriority(); };
  static unsigned long getDefaultRedrawPriority()  { return SoSceneManager::getDefaultRedrawPriority(); };

  void render()  { sceneManager->render(clearColorBuffer, clearZBuffer); };
  void scheduleRedraw()  { sceneManager->scheduleRedraw(); };

  void setGLRenderAction(SoGLRenderAction *ra)  { sceneManager->setGLRenderAction(ra); };
  SoGLRenderAction* getGLRenderAction() const  { return sceneManager->getGLRenderAction(); };

  void setSceneManager(SoSceneManager *sm);
  SoSceneManager* getSceneManager() const  { return sceneManager; };

  // from SoXtGLWidget
  void setBorder(SbBool onOrOff);
  SbBool isBorder() const  { return border; }

  void setDoubleBuffer(SbBool onOrOff);
  SbBool isDoubleBuffer()  { return doubleBuffer; }

  // from SoXtComponent
  virtual void show();
  virtual void hide();
  SbBool isVisible()  { return visible && SoSDL::widgetVisible; };

  void setSize(const SbVec2s &size);
  SbVec2s getSize() const;

  void setTitle(const char *newTitle);
  const char* getTitle() const  { return title; };

  void setIconTitle(const char *newIconTitle);
  const char* getIconTitle() const  { return iconTitle; }

  void setWindowCloseCallback(void (*func)(void *userData, SoSDLRenderArea *ra), void *data = NULL)  { closeCallback = func; closeCBData = data; }
  
  Widget getWidget() const  { return handle; };
  static SoSDLRenderArea* getComponent(Widget w)  { return SoSDL::getWindowFromHandle(w); }


protected:

  SDL_Surface* handle;
  SoSDLRenderArea *nextWindowsListItem;
  SbVec2s winSize;
  int fsWidth, fsHeight;
  int fsBPP;
  SbBool acceptMouseInput;
  SbBool acceptKeyboardInput;
  char *title;                           
  char *iconTitle;
  bool border;

  void renderCallback();
  static void handleRender();
  static void sceneManagerRenderCallback(void *userData, SoSceneManager *mgr);
  void sizeChanged(int w, int h);
  static void handleReshape(int w, int h);

  int mouseX;
  int mouseY;
  void makeEvent(SoEvent *event);
  static void handleKeyboard(int key, bool down);
  static void handleMouseMotion(int x, int y);
  static void handleMouseButton(bool down, int button, int x, int y);
  
  void (*closeCallback)(void *userData, SoSDLRenderArea *ra);
  void *closeCBData;
  static bool handleClose();

  SoSceneManager *sceneManager;
  bool ownedSceneManager;

  bool doubleBuffer;
  bool autoRedraw;

  bool visible;
  bool fullScreen;
  SbVec2s position;
  SbBool clearColorBuffer;
  SbBool clearZBuffer;

  virtual void doShow();
  virtual void doHide();

  void createWindow();
  void destroyWindow();
  void recreateWindow(SbVec2s size);

};

#endif /* SO_SDL_RENDER_AREA_H_ */



